/****
**
**  Depth Of Field Calculator
**  Application Code by Norbert Frst, 2005
**  All Rights reserved.
**
**  Vs.1.0. Release date 06/27/2005
**  Vs.1.1. Release date 07/04/2005
**  Vs.1.2. Release Date 06/08/2005
**  Vs.1.3. Release date 25/08/2005
**  Vs.1.5. Various film formats added
**
**  ToDo List:
**          Field for DoLC / manual entry
**          Graphics rework
**          Proper background for Done button
**          Localization
**          Preferences
**
****/

var feetPerMeter = 0.3048;
var lens;          
var isFireFox = false;
var isLogging = false;

function log(message) {
    if (isLogging) {
        if (isFireFox) {
            alert (message);
        } else {
            window.console.log (message);
        }
    }
}


function objektiv() {
    this.brennweite = 20;  
    this.korrektur =  1.52;  
    this.streuung = 0.019;   
    this.blende = 8;         
    this.distanz = {value:0.0, feld:0 };     
    this.hf = 0.0;
    this.nf = 0.0; 
    this.ff = 0.0; 
        
    this.isMetric = true;   /* bestimmt, ob ft. oder m angezeigt sind */
    
    /** die Funktionen **/
        
    this.hyperfokus = function() {
        var wBrennW = this.brennweite * this.korrektur;
        this.hf = (  (wBrennW * wBrennW) 
                / (this.blende * this.streuung)) 
            + wBrennW;
        return (this.toGUI(this.hf));
    }

    this.fernerFokus = function() {
        var wBrennW = this.brennweite * this.korrektur;
        this.ff = (this.distanz.value * (this.hf - wBrennW))
                / (this.hf - this.distanz.value);
        return (this.toGUI(this.ff));
    }

    this.naherFokus = function() {
        var wBrennW = this.brennweite * this.korrektur;
        this.nf = (this.distanz.value * (this.hf - wBrennW))
                / (this.hf + this.distanz.value - 2 * wBrennW);
        return (this.toGUI(this.nf));
    }
    
    this.getBrennweite = function(feld) {
        var w = this.checkNumber(feld);
        if (isNaN(w))
            return false;
        if (w != this.brennweite) {
            this.brennweite = w;
            this.recalc();
        }
        return true;
    }

    this.getKorrektur = function(feld) {
        var w = this.checkNumber(feld);
        if (isNaN(w))
            return false;
        if (this.korrektur != w) {
            this.korrektur = w;
            this.recalc();
        }
        return true;
    }

    this.getStreuung = function(feld) {
        var w = this.checkNumber(feld);
        if (isNaN(w))
            return false;
        /** Der Wert wird als inch angegeben 
        if (!this.isMetric) { 
            w *= 25.4;    
        } ***/
        if (w != this.streuung) {
            this.streuung = w;
            this.recalc();
        }
        return true;
    }

    this.getBlende = function(wert) {
        this.blende = wert;
        if (isLogging)
            log ("Blende: " + this.blende);
        this.recalc();
    }

    this.getDistanz = function(feld) {
        this.distanz.feld = feld;
        var w = this.checkNumber(feld);
        if (isNaN(w))
            return false;
        if (this.isMetric) {
            w = w * 1000.0;
        } else {
            w = w * feetPerMeter * 1000.0;
        }
        if (w != this.distanz.value) {
            this.distanz.value = w;
            this.recalc();
        }
        return true;
    }

    this.getMetric = function (bValue) { 
        var bState = bValue == "true" ? true : false;
        if (this.isMetric != bState) {
            this.isMetric = bState;
            if (this.distanz.value != 0.0) {
                this.distanz.feld.value = this.toGUI(this.distanz.value);   
                this.recalc();
            }
        }
    }
    /** Wenn etwas ans GUI geht, muss aus den internen mm Meter oder Fuss werden **/
    this.toGUI = function(wert) {
        if (!this.isMetric) {
            var a = wert / 100.0 / feetPerMeter;
            return Math.round(a) / 10.0;
        } 
        return Math.round(wert / 100.0) / 10.0;
    }

    this.recalc = function() {
        var n;
        document.getElementById('pHF').innerText = this.hyperfokus();
   
        if (isLogging)
            log ("Recalc hf: " + this.hyperfokus() );
        if (this.distanz.value != 0.0) { 
            var nF = this.naherFokus();
            document.getElementById('pNF').innerText = nF;
            n = this.toGUI(this.distanz.value);
            n = Math.abs(n - nF);        
            n = Math.round(n * 10.0) / 10.0; 
            document.getElementById('pdofN').innerText = n;
        
            n = this.fernerFokus();    
            if (n < 0) {
                document.getElementById('pFF').innerText = "INF.";
                document.getElementById('pdofF').innerText= "INF.";
            } else {
                document.getElementById('pFF').innerText = n;
                n = n - this.toGUI(this.distanz.value);    
                n = Math.round(n * 10.0) / 10.0; 
                document.getElementById('pdofF').innerText = n;                
            }
        }
    }

    this.checkNumber = function(feld) {
        var w = parseFloat(feld.value);
        if (isNaN(w)) {
    	   feld.focus();
    	   feld.select();
        }
        return Math.abs(w);
    }
}


/* Mit setup wird eine Instanz erzeugt */
function setup() {
    lens = new objektiv ();
    /** Wenn das Widget inaktiv ist... 
    if (window.widget) {
        widget.onshow = onshow;
        widget.onhide = onhide;
    }
    **/
    /** hier werden die preferences gehandelt... **/
           
}

/****
**
**  Depth Of Field Calculator
**  Widget Code by Norbert Frst, 2005
**  All Rights reserved.
**
****/


function popupBlendeChanged(elem) {
    var chosenOption = elem.options[elem.selectedIndex].value;
    document.getElementById("blendenWert").innerText = chosenOption;
    lens.getBlende(chosenOption);
}
        
function onshow() {
// Hier kommt Code, der dem widget Aktivitt zulsst
}

function onhide() {
// wenn nicht gezeigt, keine Aktivitt...
}
        
        
function showBack(event) { 
    var front=document.getElementById("front");
    var back=document.getElementById("back");
    document.getElementById('fliprollie').style.display = 'none';
   
    if (window.widget)
        widget.prepareForTransition("ToBack");
                
    front.style.display="none";
    back.style.display="block";
            
    if (window.widget)
        setTimeout ('widget.performTransition();', 0);
}

function hideBack(event) {
    var front=document.getElementById("front");
    var back=document.getElementById("back");
            
    if (window.widget)
        widget.prepareForTransition("ToFront");
                
    back.style.display="none";
    front.style.display="block";

            
    if (window.widget)
        setTimeout ('widget.performTransition();', 0);
}

        
var flipShown = false;
var animation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, 
                        firstElement:null, timer:null };
                        
function mousemove(event) {
    if (!flipShown) {
        if (animation.timer != null) {
            clearInterval (animation.timer);
            animation.timer = null;
        }
        var starttime = (new Date).getTime() - 13;
        animation.duration = 500;
        animation.starttime = starttime;
        animation.firstElement=document.getElementById('flip');
        animation.timer = setInterval ("animate();", 13);
        animation.from = animation.now;
        animation.to = 1.0;
        animate();
        flipShown = true;
    }
}
        
function mouseexit(event) {
    if (flipShown) {
        if (animation.timer != null) {
            clearInterval (animation.timer);
            animation.timer = null;
        }
        var starttime = (new Date).getTime() - 13;
        animation.duration = 500;
        animation.starttime = starttime;
        animation.firstElement=document.getElementById('flip');
        animation.timer = setInterval ("animate();", 13);
        animation.from = animation.now;
        animation.to = 0.0;
        animate();
        flipShown = false;
    }
}
        
function animate() {
    var T;
    var ease;
    var time = (new Date).getTime();
            
    T = limit_3(time-animation.starttime, 0, animation.duration);
      
    if (T >= animation.duration) {
        clearInterval (animation.timer);
        animation.timer = null;
        animation.now = animation.to;
    } else {
        ease = 0.5 - (0.5 * Math.cos(Math.PI * T / animation.duration));
        animate.now = computeNextFloat (animation.from, animation.to, ease);
    }
    animation.firstElement.style.opacity = animate.now;
}
        
function limit_3 (a, b, c) {
    return a < b ? b : (a > c ? c : a);
}
        
function computeNextFloat( from, to, ease) {
    return from + (to - from) * ease;
}
        
function enterflip (event) {
    document.getElementById('fliprollie').style.display = 'block';
}
        
function exitflip (event) {
    document.getElementById('fliprollie').style.display = 'none';
}
